{This Component is a enhanced combobox like seen on a famous cad program.
It is based on the SKDLIST.ZIP by robert@alpine.net
You can add up to 8 "toggle" bitmaps,stored in a TImageList.
The status of these buttons is stored in an object within the CadComboBox.
With double clicking on any of the bitmaps, you can change their status.
****************************
New properties:
Imagelist:Holds the bitmaps. The bitmap with index zero (0) will
          be seen if the according flag is set to false.
          Image 1..Image 7 are the images of the according flags.
          Note, that if property ShowEmptyFlags=false you wont see Bitmap Index 0!

FlagCount:Here you select the number of toggle buttons (0..8). If this value is bigger
          than the maximum Imaglist index you may see nothing.

ShowEmptyFlag:See above
*****************************
New Events:
OnFlagClick:Whenever you click on a bitmap, it toggles and this event is fired.
*****************************
New Procedures/functions:
procedure AddEntry:Since the flag values are stored in attached objects of the
                   the tstrings object you may NOT use CadlistBox.items.add('xxxxx'). Instead,
                   you should use AddEntry(s:string;flags:byte);

function GetFlag(Index:integer;FlagNumber:byte):boolean
         Get the flag status of the given index.

procedure SetFlag(Index:integer;Flagnumber:byte;Flag:boolean);
          Sets one single flag within the tstrings object.

For further details look into the code. Just go down!

To use the component:
Drop a TlistBox and fill it with up to eight pictures (do not use too big ones)
Drop the CadComboBox and fill the Imagelist property.
Add strings programmatically with the AddEntry-procedure. Do not use items.add etc.
Check/Change the flags of the strings with the mouse or with Get/Setflag.
Thats it.
*****************************

(p)1998 by Bernhard Knibbe.

This component is freeware.
Do with it whatever you want, but dont blaim me for anything.
Thanks to all who offer free components to the Delphi Super Page!!!

Bernhard Knibbe
Vienna, Austria
b_knibbe@csi.com
http://ourworld.compuserve.com/homepages/b_knibbe
Please report any bugs/problems/improvements!
}

unit CadComboBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, DsgnIntf, StdCtrls,math;

type TFlagClickEvent=procedure(Sender:Tobject;Index:integer;FlagNumber:byte;Flag:boolean) of object;

     TCadComboBox = class(TComboBox)
  private
    FShowEmptyFlag:boolean;
    FFlagCount:byte;
    FMargin:byte;
    FImageIndex : Integer;
    FimageList : TImageList;
    FCadComboBox : TCadComboBox;
    FOnFlagClick:TFlagClickEvent;
    Function GetImageList : TImageList;
    Procedure SetImageList(Value : TImageList);
    Function GetImageIndex : Integer;
    Procedure DrawImage(Index, NewIndex : Integer; Rect: TRect);
    procedure SetFlagCount(AFlagCount:byte);
    procedure SetMargin(AMargin:byte);
    Procedure SetEmptyFlag(AFlag:boolean);
  protected
    procedure MouseDown(Button: TMouseButton;Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    Procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);  override;
    Function AddEntry(s:string;flag:byte):integer;
    Property NewImageIndex : Integer read FImageIndex write FImageIndex Stored True;
    published
    property ImageList : TImageList read GetImageList write SetImageList Stored True;
    Property FlagCount:byte read FFlagCount write SetFlagCount Stored True;
    Property Margin:byte read FMargin write SetMargin Stored True;
    Property ShowEmptyFlag:boolean read fShowEmptyFlag write SetEmptyFlag Stored True;
    Property OnFlagClick:TFlagClickEvent read fOnFlagClick write fOnFlagClick;
    Function GetFlag(Index:integer;FlagNumber:byte):boolean;
    Procedure SetFlag(Index:integer;FlagNumber:byte;Flag:Boolean);
  end;

procedure Register;

implementation
{$R *.dcr}

function test(input,which:byte):byte;
begin
 if input and (1 shl (which)) = 0 then test:=0 else test:=1;
end;

Function TCadComboBox.AddEntry(s:string;flag:byte):integer;
begin
Items.addobject(s,TObject(makelong(flag,0)));
end;

procedure TCadComboBox.SetFlagCount(AFlagCount:byte);
begin
if AFlagCount<9 then if AFlagCount<>FFlagCount then FFLagCount:=AFlagCount;
refresh;
end;

Function TCadComboBox.GetFlag(Index:integer;FlagNumber:byte):boolean;
begin
result:=boolean(test(loword(Items.Objects[index]),FlagNumber));
end;

Procedure TCadComboBox.SetFlag(Index:integer;FlagNumber:byte;Flag:Boolean);
var x:integer;
begin
x:=1 shl Flagnumber;
if flag<>getflag(index,FlagNumber) then
Items.objects[Index]:=tobject(makelong(loword(Items.Objects[index]) xor x,0));
refresh;
end;

procedure TCadComboBox.SetMargin(AMargin:byte);
begin
if AMargin<10 then if AMargin<>FMargin then FMargin:=AMargin;
refresh;
end;

Procedure TCadComboBox.SetEmptyFlag(AFlag:boolean);
begin
if FShowEmptyFlag<>AFlag then FShowEmptyFlag:=AFlag;
refresh;
end;

Procedure TCadComboBox.DrawImage(Index, NewIndex : Integer; Rect: TRect);
 Var
 NewBitMap : TBitmap;
 x:integer;
 TheFlag:byte;
 Size:byte;
Begin
  NewBitMap := TBitMap.Create;
    With Canvas Do
     Begin
        FimageList.GetBitmap(0, NewBitmap);
        size:=newbitmap.width;
        newbitmap.assign(nil);
        FillRect(Rect);
        for x:=0 to FFLagCount-1 do begin
         TheFlag:=test(loword(Items.Objects[index]),x);
         if x>0 then TheFlag:=TheFlag*x;
         if fShowEmptyFlag then begin
            FimageList.GetBitmap(TheFlag, NewBitmap);
         end else begin
            if theflag>0 then
            FimageList.GetBitmap(TheFlag, NewBitmap);
         end;
         Draw((x*newbitmap.width+x*FMargin)+Rect.Left, Rect.Top+0, NewBitmap);
         newbitmap.assign(nil);
        end;
         TextOut((FFlagCount*size+(FFLagCount+1)*FMargin)+Rect.Left, Rect.Top + 0,
         Items.strings[Index]);

     End;
   NewBitMap.Free;
End;

Procedure TCadComboBox.SetImageList(Value : TImageList);
Begin
  If Value <> FimageList Then
    FimageList := Value;
If Not Assigned(FimageList) Then
  FimageList := Nil;
End;

Function TCadComboBox.GetImageList : TImageList;
Begin
 Result := FimageList;
End;

Function TCadComboBox.GetImageIndex : Integer;
Begin
 Result := FImageIndex;
End;

procedure TCadComboBox.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
 Inherited;
  DrawImage(Index, FImageIndex, Rect);
end;

procedure TCadComboBox.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var b:boolean;
    FlagNumber:byte;
begin
 inherited MouseDown(button,shift,x,y);
 if x<fflagcount*(fimagelist.width+fmargin) then begin
  droppeddown:=false;
  FlagNumber:=(x div (fimagelist.width+fmargin));
  b:=getflag(itemindex,FlagNumber);
     setflag(itemindex,FlagNumber,not b);
 end;
 if assigned(fOnFlagClick) then fOnFlagClick(self,itemindex,flagnumber,not b);
end;

constructor TCadComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Style := csOwnerDrawFixed;
  ControlStyle:= ControlStyle + [csAcceptsControls];
  FFLagCount:=1;
end;

Destructor TCadComboBox.Destroy;
Begin
 inherited Destroy;
End;

procedure Register;
begin
  RegisterComponents('Freeware', [TCadComboBox]);

end;

end.
